@@ -96,6 +96,10 @@ sub _db_connection
 	$sth->execute or $self->logit("FATAL: " . $dbh->errstr . "\n", 0, 1);
 	$sth->finish;
 
+	# Set the date format
+	#$dbh->do("SET LANGUAGE us_english") or $self->logit("FATAL: " . $dbh->errstr . "\n", 0, 1);
+	$dbh->do("SET DATEFORMAT dmy") or $self->logit("FATAL: " . $dbh->errstr . "\n", 0, 1);
+
 	# Force execution of initial command
 	$self->_ora_initial_command($dbh);
 
@@ -168,7 +172,7 @@ sub _table_exists
 
 =head2 _get_encoding
 
-This function retrieves the Oracle database encoding
+This function retrieves the MSSQL database encoding
 
 Returns a handle to a DB query statement.
 
@@ -245,7 +249,7 @@ sub _table_info
 
         my $schema_clause = '';
         $schema_clause = " AND s.name='$self->{schema}'" if ($self->{schema});
-	my $sql = qq{SELECT t.NAME AS TABLE_NAME, NULL AS comment, t.type_desc as TABLE_TYPE, p.rows AS RowCounts, SUM(a.used_pages)  * 8 / 1024 AS UsedSpaceMB, CONVERT(DECIMAL,SUM(a.total_pages)) * 8 / 1024 AS TotalSpaceMB, s.Name AS TABLE_SCHEMA, SCHEMA_NAME(t.principal_id), i.type_desc
+	my $sql = qq{SELECT t.NAME AS TABLE_NAME, NULL AS comment, t.type_desc as TABLE_TYPE, p.rows AS RowCounts, SUM(a.used_pages)  * 8 / 1024 AS UsedSpaceMB, CONVERT(DECIMAL,SUM(a.total_pages)) * 8 / 1024 AS TotalSpaceMB, s.Name AS TABLE_SCHEMA, SCHEMA_NAME(t.principal_id), i.type_desc, p.data_compression_desc
 FROM sys.tables t
 INNER JOIN sys.indexes i ON t.OBJECT_ID = i.object_id
 INNER JOIN sys.partitions p ON i.object_id = p.OBJECT_ID AND i.index_id = p.index_id
@@ -256,7 +260,7 @@ WHERE t.is_ms_shipped = 0 AND i.OBJECT_ID > 255 AND t.type='U' AND t.NAME NOT LI
 	my %tables_infos = ();
 	my %comments = ();
 	$sql .= $self->limit_to_objects('TABLE', 't.Name');
-	$sql .= " GROUP BY t.type_desc, i.type_desc, s.Name, t.Name, SCHEMA_NAME(t.principal_id), p.Rows ORDER BY s.Name, t.Name";
+	$sql .= " GROUP BY t.type_desc, i.type_desc, s.Name, t.Name, SCHEMA_NAME(t.principal_id), p.Rows, p.data_compression_desc ORDER BY s.Name, t.Name";
 	my $sth = $self->{dbh}->prepare( $sql ) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	while (my $row = $sth->fetch)
@@ -277,7 +281,8 @@ WHERE t.is_ms_shipped = 0 AND i.OBJECT_ID > 255 AND t.type='U' AND t.NAME NOT LI
 		$tables_infos{$row->[0]}{auto_increment} = 0;
 		$tables_infos{$row->[0]}{tablespace} = $tbspname{$row->[0]} || '';
 		$tables_infos{$row->[0]}{partitioned} = 1 if (exists $self->{partitions_list}{"\L$row->[0]\E"});
-		$tables_infos{$row->[0]}{index_type} = $row->[8];
+		$tables_infos{$row->[0]}{index_type} = $row->[8] if ($row->[8] =~ /^(XML|SPATIAL|.*COLUMNSTORE)$/i);
+		$tables_infos{$row->[0]}{compressed} = $row->[9];
 	}
 	$sth->finish();
 
@@ -341,11 +346,14 @@ sub _column_info
     NULL as AUTO_INCREMENT,
     NULL AS ENUM_INFO,
     object_definition(c.rule_object_id),
-    t.is_user_defined
+    t.is_user_defined,
+    m.is_masked,
+    m.masking_function
 FROM sys.columns c
 INNER JOIN sys.types t ON t.user_type_id = c.user_type_id
 INNER JOIN sys.tables AS tb ON tb.object_id = c.object_id
 INNER JOIN sys.schemas AS s ON s.schema_id = tb.schema_id
+LEFT JOIN sys.masked_columns AS m ON m.object_id = tb.object_id AND m.column_id = c.column_id
 $condition
 ORDER BY c.column_id};
 
@@ -406,15 +414,19 @@ sub _get_indexes
 
 	# When comparing number of index we need to retrieve generated index (mostly PK)
 	my $generated = '';
-	$generated = " AND Id.auto_created = 0" if (!$generated_indexes);
+	my $col_generated = "''";
+	if ($self->{db_version} !~ /SQL Server 201[0-6]/) {
+		$generated = " AND Id.auto_created = 0" if (!$generated_indexes);
+		$col_generated = 'Id.auto_created';
+	}
 
 	my $t0 = Benchmark->new;
 	my $sth = '';
-	my $sql = qq{SELECT Id.name AS index_name, AC.name AS column_name, Id.is_unique AS UNIQUENESS, AC.column_id AS COLUMN_POSITION, Id.type AS INDEX_TYPE, 'U' AS TABLE_TYPE, Id.auto_created AS GENERATED, NULL AS JOIN_INDEX, t.name AS TABLE_NAME, s.name as TABLE_SCHEMA, Id.data_space_id AS TABLESPACE_NAME, Id.type_desc AS ITYP_NAME, Id.filter_definition AS PARAMETERS, IC.is_descending_key AS DESCEND, id.is_primary_key PRIMARY_KEY, typ.name AS COL_TYPE_NAME
+	my $sql = qq{SELECT Id.name AS index_name, AC.name AS column_name, Id.is_unique AS UNIQUENESS, AC.column_id AS COLUMN_POSITION, Id.type AS INDEX_TYPE, 'U' AS TABLE_TYPE, $col_generated AS GENERATED, NULL AS JOIN_INDEX, t.name AS TABLE_NAME, s.name as TABLE_SCHEMA, Id.data_space_id AS TABLESPACE_NAME, Id.type_desc AS ITYP_NAME, Id.filter_definition AS PARAMETERS, IC.is_descending_key AS DESCEND, id.is_primary_key PRIMARY_KEY, typ.name AS COL_TYPE_NAME, IC.is_included_column
 FROM sys.tables AS T
-INNER JOIN sys.indexes Id ON T.object_id = Id.object_id 
-INNER JOIN sys.index_columns IC ON Id.object_id = IC.object_id
-INNER JOIN sys.all_columns AC ON T.object_id = AC.object_id AND IC.column_id = AC.column_id 
+INNER JOIN sys.indexes Id ON T.object_id = Id.object_id
+INNER JOIN sys.index_columns IC ON Id.object_id = IC.object_id AND Id.index_id = IC.index_id
+INNER JOIN sys.all_columns AC ON IC.object_id =  AC.object_id AND IC.column_id = AC.column_id
 INNER JOIN sys.types typ ON typ.user_type_id = AC.user_type_id
 LEFT OUTER JOIN sys.schemas s ON t.schema_id = s.schema_id
 WHERE T.is_ms_shipped = 0 $generated $condition
@@ -448,12 +460,20 @@ ORDER BY T.name, Id.index_id, IC.key_ordinal
 		if ( !$self->{indexes_renaming} && !$self->{indexes_suffix} && (lc($row->[0]) eq lc($table)) ) {
 			 print STDERR "WARNING: index $row->[0] has the same name as the table itself. Please rename it before export or enable INDEXES_RENAMING.\n";
 		}
-		$unique{$row->[8]}{$row->[0]} = $row->[2];
 
 		# Save original column name
 		my $colname = $row->[1];
 		# Quote column with unsupported symbols
 		$row->[1] = $self->quote_object_name($row->[1]);
+		# Covered columns (include)
+		if ($row->[16] == 1)
+		{
+			push(@{$idx_type{$row->[8]}{$row->[0]}{type_include}}, $row->[1]);
+			next;
+		}
+
+		$unique{$row->[8]}{$row->[0]} = $row->[2];
+
 		# Replace function based index type
 		if ( $row->[13] )
 		{
@@ -466,22 +486,6 @@ ORDER BY T.name, Id.index_id, IC.key_ordinal
 		$idx_type{$row->[8]}{$row->[0]}{type_name} = $row->[11];
 		$idx_type{$row->[8]}{$row->[0]}{type} = $row->[4];
 
-#		my $idx_name = $row->[0];
-#		if (!$self->{schema} && $self->{export_schema}) {
-#			$idx_name = "$row->[9].$row->[0]";
-#		}
-#		if ($row->[11] =~ /SPATIAL_INDEX/) {
-#			$idx_type{$row->[8]}{$row->[0]}{type} = 'SPATIAL INDEX';
-#			if ($row->[12] =~ /layer_gtype=([^\s,]+)/i) {
-#				$idx_type{$row->[9]}{$row->[0]}{type_constraint} = uc($1);
-#			}
-#			if ($row->[12] =~ /sdo_indx_dims=(\d+)/i) {
-#				$idx_type{$row->[8]}{$row->[0]}{type_dims} = $1;
-#			}
-#		}
-#		if ($row->[4] eq 'BITMAP') {
-#			$idx_type{$row->[8]}{$row->[0]}{type} = $row->[4];
-#		}
 		push(@{$data{$row->[8]}{$row->[0]}}, $row->[1]);
 		$index_tablespace{$row->[8]}{$row->[0]} = $row->[10];
 		$nidx++;
@@ -540,16 +544,28 @@ sub _foreign_key
 
         my $condition = '';
         $condition .= " AND OBJECT_NAME (f.referenced_object_id) = '$table' " if ($table);
-        $condition .= " AND SCHEMA_NAME(f.schema_id) = '$self->{schema}' " if ($self->{schema});
+        $condition .= " AND SCHEMA_NAME(t_parent.schema_id) = '$self->{schema}' " if ($self->{schema});
 	$condition =~ s/^ AND / WHERE /;
 
         my $deferrable = $self->{fkey_deferrable} ? "'DEFERRABLE' AS DEFERRABLE" : "DEFERRABLE";
-	my $sql = qq{SELECT f.name ConsName, SCHEMA_NAME(f.schema_id) SchemaName, COL_NAME(fc.parent_object_id,fc.parent_column_id) ColName, OBJECT_NAME(f.parent_object_id) TableName, t.name as ReferencedTableName, COL_NAME(f.referenced_object_id, key_index_id) as ReferencedColumnName,update_referential_action_desc UPDATE_RULE, delete_referential_action_desc DELETE_RULE, SCHEMA_NAME(t.schema_id)
-FROM sys.foreign_keys AS f
-INNER JOIN sys.foreign_key_columns AS fc ON f.OBJECT_ID = fc.constraint_object_id
-INNER JOIN sys.tables t ON t.OBJECT_ID = fc.referenced_object_id
-LEFT OUTER JOIN sys.schemas s ON f.principal_id = s.schema_id
-$condition};
+	my $sql = qq{SELECT
+    fk.name AS ConsName,
+    SCHEMA_NAME(fk.schema_id) SchemaName,
+    c_parent.name AS ParentColumnName,
+    t_parent.name AS ParentTableName,
+    t_child.name AS ReferencedTableName,
+    c_child.name AS ReferencedColumnName,
+    update_referential_action_desc UPDATE_RULE,
+    delete_referential_action_desc DELETE_RULE,
+    SCHEMA_NAME(t_parent.schema_id)
+FROM sys.foreign_keys fk
+INNER JOIN sys.foreign_key_columns fkc ON fkc.constraint_object_id = fk.object_id
+INNER JOIN sys.tables t_parent ON t_parent.object_id = fk.parent_object_id
+INNER JOIN sys.columns c_parent ON fkc.parent_column_id = c_parent.column_id AND c_parent.object_id = t_parent.object_id
+INNER JOIN sys.tables t_child ON t_child.object_id = fk.referenced_object_id
+INNER JOIN sys.columns c_child ON c_child.object_id = t_child.object_id AND fkc.referenced_column_id = c_child.column_id
+$condition
+ORDER BY t_parent.name, c_parent.name};
 
         my $sth = $self->{dbh}->prepare($sql) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
         $sth->execute or $self->logit("FATAL: " . $sth->errstr . "\n", 0, 1);
@@ -581,7 +597,7 @@ $condition};
 
 =head2 _get_views
 
-This function implements an Oracle-native views information.
+This function implements an MSSQL-native views information.
 
 Returns a hash of view names with the SQL queries they are based on.
 
@@ -759,7 +775,7 @@ ORDER BY sh.name, i.name, ic.key_ordinal;
 		}
 
 		my $idxname = $row->[3] . '_idx' . $i;
-		$idxname = $row->[2] if ($row->[2]);
+		$idxname = $row->[1] if ($row->[1]);
 		my $key_type = 'U';
 		$key_type = 'P' if ($row->[7]);
 
@@ -955,8 +971,10 @@ sub _lookup_function
         my %fct_detail = ();
         $fct_detail{func_ret_type} = 'OPAQUE';
 
+	my @code = ();
         # Split data into declarative and code part
-        ($fct_detail{declare}, $fct_detail{code}) = split(/\b(?:BEGIN|SET|SELECT|INSERT|UPDATE|IF)\b/i, $code, 2);
+        ($fct_detail{declare}, @code) = split(/\b(BEGIN|SET|SELECT|INSERT|UPDATE|IF)\b/i, $code, 3);
+	$fct_detail{code} = join(' ', @code);
 	return if (!$fct_detail{code});
 
 	# Look for table variables in code and rewrite them as temporary tables
@@ -1125,8 +1143,12 @@ sub _lookup_function
 
 	$fct_detail{args} =~ s/\s*$//s;
 	$fct_detail{args} =~ s/^\s*//s;
-	$fct_detail{code} =~ s/^[\r\n]*/\n/s;
+	$fct_detail{args} =~ s/[\s]+=[\s]+/ DEFAULT /gs;
 
+	$fct_detail{code} =~ s/^[\r\n]*/\n/s;
+	if ($fct_detail{code} !~ /\sEND$/s) {
+		$fct_detail{code} =~ s/$/\nEND/;
+	}
 
 	# Remove %ROWTYPE from return type
 	$fct_detail{func_ret_type} =~ s/\%ROWTYPE//igs;
@@ -1345,7 +1367,7 @@ sub _sql_type
 				{
 					if ($precision)
 					{
-						if ($type =~ /(TINYINT|SMALLINT|INTEGER|BIGINT|INT)/ && $precision)
+						if ($type =~ /(TINYINT|SMALLINT|INTEGER|BIGINT|INT)/)
 						{
 							if ($precision < 5) {
 								return 'smallint';
@@ -1355,28 +1377,21 @@ sub _sql_type
 								return 'bigint';
 							}
 						}
+						elsif ($type =~ /(FLOAT|REAL)/)
+						{
+							return "float($precision)";
+						}
 						return "numeric($precision)";
 					}
 					else
 					{
-						# Most of the time interger should be enought?
+						# Most of the time integer should be enought?
 						return $self->{data_type}{$type};
 					}
 				}
 				else
 				{
-					if ($precision)
-					{
-						if ($self->{pg_numeric_type})
-						{
-							if ($precision <= 6) {
-								return 'real';
-							} else {
-								return 'double precision';
-							}
-						}
-						return "decimal($precision,$scale)";
-					}
+					return "numeric($precision,$scale)";
 				}
 			}
 			$self->{use_uuid} = 1 if ($type =~ /UNIQUEIDENTIFIER/);
@@ -1457,7 +1472,12 @@ sub replace_sql_type
 						$str =~ s/\b$type\b\s*\([^\)]+\)/numeric\%\|$precision\%\|\%/i;
 					}
 				}
-				else {
+				elsif ($type =~ /(FLOAT|REAL)/)
+				{
+					return "float($precision)";
+				}
+				else
+				{
 					$str =~ s/\b$type\b\s*\([^\)]+\)/$self->{data_type}{$type}\%\|$precision\%\|\%/is;
 				}
 			}
@@ -1766,7 +1786,7 @@ LEFT OUTER JOIN sys.schemas sch ON t.schema_id = sch.schema_id
 
 =head2 _get_objects
 
-This function retrieves all object the Oracle information
+This function retrieves all object the MSSQL information
 
 =cut
 
@@ -1818,7 +1838,11 @@ sub _get_objects
 	# INDEX
 	foreach my $t (@{$infos{TABLE}})
 	{
-		my $sql = "SELECT Id.name AS index_name FROM sys.tables AS T INNER JOIN sys.indexes Id ON T.object_id = Id.object_id LEFT OUTER JOIN sys.schemas s ON t.schema_id = s.schema_id WHERE T.is_ms_shipped = 0 AND Id.auto_created = 0 AND OBJECT_NAME(Id.object_id, DB_ID())='$t' AND Id.is_primary_key = 0";
+		my $auto_generated = '';
+		if ($self->{db_version} !~ /SQL Server 201[0-6]/) {
+			$auto_generated = ' AND Id.auto_created = 0';
+		}
+		my $sql = "SELECT Id.name AS index_name FROM sys.tables AS T INNER JOIN sys.indexes Id ON T.object_id = Id.object_id LEFT OUTER JOIN sys.schemas s ON t.schema_id = s.schema_id WHERE T.is_ms_shipped = 0 AND OBJECT_NAME(Id.object_id, DB_ID())='$t' AND Id.is_primary_key = 0$auto_generated";
 		if (!$self->{schema}) {
 			$sql .= " AND s.name NOT IN ('" . join("','", @{$self->{sysusers}}) . "')";
 		} else {
@@ -1998,7 +2022,7 @@ FROM sys.master_files m JOIN sys.databases d ON d.database_id = m.database_id an
 
 =head2 _get_largest_tables
 
-This function retrieves the list of largest table of the Oracle database in MB
+This function retrieves the list of largest table of the MSSQL database in MB
 
 =cut
 
