@@ -10,7 +10,7 @@ use POSIX qw(locale_h);
 setlocale(LC_NUMERIC,"C");
 
 
-$VERSION = '24.3';
+$VERSION = '25.0';
 
 # Some function might be excluded from export and assessment.
 our @EXCLUDED_FUNCTION = ('SQUIRREL_GET_ERROR_OFFSET');
@@ -1085,6 +1085,41 @@ sub _lookup_function
 
 	($fct_detail{code}, $fct_detail{declare}) = replace_mysql_variables($self, $fct_detail{code}, $fct_detail{declare});
 
+	# Try to replace LEAVE label by EXIT label
+	my %repl_leave = ();
+	my $i = 0;
+	while ($fct_detail{code} =~ s/LEAVE\s+([^\s;]+)\s*;/%REPEXITLBL$i%/igs)
+	{
+		my $label = $1;
+		if ( $fct_detail{code} =~ /$label:/is) {
+			$repl_leave{$i} = "EXIT $label;";
+		} else {
+			# This is a main block label
+			$repl_leave{$i} = "RETURN;";
+		}
+		$fct_detail{code} =~ s/$label:/<<$label>>/igs;
+		$i++;
+	}
+	foreach $i (keys %repl_leave) {
+		$fct_detail{code} =~ s/\%REPEXITLBL$i\%/$repl_leave{$i}/gs;
+	}
+	%repl_leave = ();
+
+	# Try to replace ITERATE label by CONTINUE label
+	my %repl_iterate = ();
+	$i = 0;
+	while ($fct_detail{code} =~ s/\bITERATE\s+([^\s;]+)\s*;/%REPITERLBL$i%/igs)
+	{
+		my $label = $1;
+		$repl_iterate{$i} = "CONTINUE $label;";
+		$fct_detail{code} =~ s/$label:/<<$label>>/igs;
+		$i++;
+	}
+	foreach $i (keys %repl_iterate) {
+		$fct_detail{code} =~ s/\%REPITERLBL$i\%/$repl_iterate{$i}/gs;
+	}
+	%repl_iterate = ();
+
 	# Remove %ROWTYPE from return type
 	$fct_detail{func_ret_type} =~ s/\%ROWTYPE//igs;
 
@@ -1096,7 +1131,8 @@ sub replace_mysql_variables
 	my ($self, $code, $declare) = @_;
 
 	# Look for mysql global variables and add them to the custom variable list
-	while ($code =~ s/\b(?:SET\s+)?\@\@(?:SESSION\.)?([^\s:=]+)\s*:=\s*([^;]+);/PERFORM set_config('$1', $2, false);/is) {
+	while ($code =~ s/\b(?:SET\s+)?\@\@(?:SESSION\.)?([^\s:=]+)\s*:=\s*([^;]+);/PERFORM set_config('$1', $2, false);/is)
+	{
 		my $n = $1;
 		my $v = $2;
 		$self->{global_variables}{$n}{name} = lc($n);
@@ -1116,7 +1152,8 @@ sub replace_mysql_variables
 
 	my @to_be_replaced = ();
 	# Look for local variable definition and append them to the declare section
-	while ($code =~ s/SET\s+\@([^\s:]+)\s*:=\s*([^;]+);/SET $1 = $2;/is) {
+	while ($code =~ s/SET\s+\@([^\s:]+)\s*:=\s*([^;]+);/SET $1 = $2;/is)
+	{
 		my $n = $1;
 		my $v = $2;
 		# Try to set a default type for the variable
@@ -1134,7 +1171,8 @@ sub replace_mysql_variables
 	}
 
 	# Look for local variable definition and append them to the declare section
-	while ($code =~ s/(\s+)\@([^\s:=]+)\s*:=\s*([^;]+);/$1$2 := $3;/is) {
+	while ($code =~ s/(\s+)\@([^\s:=]+)\s*:=\s*([^;]+);/$1$2 := $3;/is)
+	{
 		my $n = $2;
 		my $v = $3;
 		# Try to set a default type for the variable
@@ -1157,7 +1195,8 @@ sub replace_mysql_variables
 	}
 
 	# Look for local variable definition and append them to the declare section
-	while ($code =~ s/\@([a-z0-9_]+)/$1/is) {
+	while ($code =~ s/\@([a-z0-9_]+)/$1/is)
+	{
 		my $n = $1;
 		# Try to set a default type for the variable
 		my $type = 'varchar';
@@ -1176,6 +1215,11 @@ sub replace_mysql_variables
 	# Look for variable definition with SELECT statement
 	$code =~ s/\bSET\s+([^\s=]+)\s*=\s*([^;]+\bSELECT\b[^;]+);/$1 = $2;/igs;
 
+	# Fix mysql DECLARE clause after BEGIN clause
+	while ($code =~ s/(BEGIN\s+)DECLARE(\s+[^\;]+;)/$1/) {
+		$declare .= "$2\n";
+	}
+
 	return ($code, $declare);
 }
 
@@ -1301,7 +1345,6 @@ sub _sql_type
 					}
 					else
 					{
-						# Most of the time interger should be enought?
 						return $self->{data_type}{$type};
 					}
 				}
